/*
**++
**  FACILITY:  csv2gif
**
**  MODULE DESCRIPTION:
**
**      Convert a "comma separated value" Polycenter Performance Advisor file
**	to a GIF file (Therefore you need PSPA 8-)).
**	This program uses the gd (GifDraw) library written by Tom Boutell
**	(boutell@netcom.com) from THE QUEST CENTER AT COLD SPRING HARBOR LABS.
**	Therefore you also need the gd library (see the URL
**	http://siva.cshl.org/gd/gd.html) to compile and link.
**
**	This program has been tested with OpenVMS VAX 6.2, DEC C V5.0-003,
**	PSPA V2.1-48 and gd 1.1.1
**
**
**  AUTHORS:
**
**      Emmanuel Adeline, adeline_e@decus.fr
**
**  CREATION DATE:  4 Aug 1995
**
**  DESIGN ISSUES:
**
**      {@tbs@}
**
**  [@optional module tags@]...
**
**  MODIFICATION HISTORY:
**
**      {@tbs@}...
**--
*/
#include "gd.h"
#include "gdfontl.h"
#include "gdfonts.h"
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#define RGB_RED 0xFF,0x00,0x00
#define RGB_YELLOW 0xFF,0xFF,0x33
#define RGB_BROWN 0xCC,0x99,0x66
#define RGB_GREEN 0x66,0xFF,0x33
#define RGB_BLUE 0x00,0x33,0xFF
#define RGB_ROSE 0xFF,0xCC,0x99
#define BOXSIZE 12
#define MAXLINE 80
#define MAXCSV 9
#define EDGE 80
#define XMAX 480
#define YMAX 300
#define NORMX(x) x
#define NORMY(y) (YMAX+EDGE-(y))
#define XZERO (2*EDGE/3)
#define YZERO (EDGE/2)
#define CSV1MAX (24*60)
#define CSV2MAX 100
#define debug 0
#define INIT -1

int main(argc, argv) 
	int argc;
	char *argv[];
{
        /* Declare the image */
        gdImagePtr im;
        FILE *csvin, *out;
        /* Declare color indexes */
        int white, black, csvcolor[MAXCSV];
	char dumchar, vmsdate[32], axis[5], csvnames[6][MAXCSV];
	char  csvline[MAXLINE]
	    , csvnode[MAXLINE]
	    , csvdate[MAXLINE]
	    , csvunit[MAXLINE]
	    , csvlegend[MAXLINE];
	int i, hour, min;
	int numcsv, xc1[MAXCSV], yc1[MAXCSV], xc2[MAXCSV], yc2[MAXCSV];
	float cpu;
	char *p;



	if (argc!=3)
	{
	    printf("Usage: csv2gif csvfile giffile\n");
	    exit(0);
	}

        /* Allocate the image: 64 pixels across by 64 pixels tall */
        im = gdImageCreate(XMAX+EDGE, YMAX+EDGE);

        /* Allocate the color white (red, green and blue all maximum).
                Since this is the first color in a new image, it will
                be the background color. */
        white = gdImageColorAllocate(im, 255, 255, 255);       
	gdImageColorTransparent(im, white);
        black = gdImageColorAllocate(im, 0, 0, 0);
         
	csvin = fopen (argv[1], "r");
	fgets (csvline,MAXLINE,csvin);
	fgets (csvnode,MAXLINE,csvin);
	fgets (csvdate,MAXLINE,csvin);
	fgets (csvunit,MAXLINE,csvin);
	fgets (csvlegend,MAXLINE,csvin);

	strtok(csvlegend,",");	/* On saute la premiere unite "time" */
	for (	numcsv=0;
		(p=strtok(NULL,","))!=NULL; 
		numcsv++ )
	{
		strcpy(csvnames[numcsv],p);
	}
	if (debug)
	{
	    printf("Numcsv = %d\n",numcsv);
	    for (i=0;  i<numcsv; printf("%s\n",csvnames[i++]));
	}

	csvcolor[0] = gdImageColorAllocate( im, RGB_RED);
	csvcolor[1] = gdImageColorAllocate( im, RGB_YELLOW);
	csvcolor[2] = gdImageColorAllocate( im, RGB_GREEN);
	csvcolor[3] = gdImageColorAllocate( im, RGB_BLUE);
	csvcolor[4] = gdImageColorAllocate( im, RGB_GREEN);
	csvcolor[5] = gdImageColorAllocate( im, RGB_ROSE);
	csvcolor[6] = gdImageColorAllocate( im, RGB_BROWN);

	for (i=0;  i<numcsv; i++)
	{
	    /* if (csvcolor[i]==-1) printf("Could not allocate color.\n"); */
	    xc1[i] = INIT;
	    yc1[i] = INIT;
	    gdImageRectangle( im, XZERO + XMAX*i/numcsv
				, NORMY(gdFontLarge->h - 1)
				, XZERO + XMAX*i/numcsv + BOXSIZE
				, NORMY(gdFontLarge->h - 1) + BOXSIZE
				, csvcolor[i] );
	    gdImageString(im, gdFontLarge
			, BOXSIZE + 5 + XZERO + XMAX*i/numcsv
			, NORMY(gdFontLarge->h)
			, csvnames[i], black);
	}


	gdImageRectangle(im,XZERO-1,YZERO-1,XZERO+XMAX+1,YZERO+YMAX+1,black);
	gdImageString(im, gdFontLarge,
			im->sx / 2- (strlen(csvnode) * gdFontLarge->w / 2),
			0,
			csvnode, black);
	gdImageString(im, gdFontLarge,
			im->sx / 2- (strlen(csvdate) * gdFontLarge->w / 2),
			gdFontLarge->h,
			csvdate, black);
	gdImageStringUp(im, gdFontSmall,
			0,
			NORMY(im->sy/2-(strlen(csvunit)*gdFontSmall->w/2)),
			csvunit, black);
	for (i=0;  i<=CSV2MAX; i+=(CSV2MAX/5))
	{
	    sprintf (axis, "%d", i);
	    gdImageString(im, gdFontLarge,
			    XZERO - 3*gdFontLarge->w - 2,
			    NORMY(YZERO + i*YMAX/CSV2MAX + gdFontLarge->h/2),
			    axis, black);
	}
	for (i=0;  i<=CSV1MAX; i+=(CSV1MAX/12))
	{
	    sprintf (axis, "%2d", i/60);
	    gdImageString(im, gdFontLarge,
			    XZERO + i*XMAX/CSV1MAX - gdFontLarge->w,
			    NORMY(YZERO-2),
			    axis, black);
	}

	while (!feof(csvin)) 
	{
	    fscanf (csvin, "%c%11s %2d:%2d%c",
			&dumchar, vmsdate, &hour, &min, &dumchar);
	    if (debug) printf("[%s] [%d] [%d]\n",vmsdate,hour,min);
	    for (i=0;  i<numcsv; i++)
	    {
		fscanf (csvin, "%c%f",&dumchar,&cpu);
		xc2[i] = xc1[i];
		yc2[i] = yc1[i];
		xc1[i] = XZERO + ( (hour*60 + min) * XMAX / CSV1MAX);
		yc1[i] = NORMY( YZERO + (int) cpu * YMAX / CSV2MAX );
		if (xc2[i]!=INIT) gdImageLine(im, xc1[i]
						, yc1[i]
						, xc2[i]
						, yc2[i]
						, csvcolor[i]);
		fscanf (csvin, " ");	/* on saute la ligne */
	    }
	}
	fclose(csvin);

        out = fopen(argv[2], "wb");
	/* gdImageInterlace(im, 1); */
        gdImageGif(im, out);      
        fclose(out);

        gdImageDestroy(im); /* Destroy the image in memory. */
}
